#include "DataLogger.h"
#include <stdarg.h>
#include "dtime.h"

DataLogger::DataLogger(string fn, int np)
{
	this->np = np;

	fp = fopen(fn.c_str(), "a");
	if(fp == NULL) {
          fprintf(stderr, "failed to open output log file\n");
          }
}

DataLogger::~DataLogger()
{
	if(fp) fclose(fp);
}

void DataLogger::Comment(string p)
{
	if(fp) fprintf(fp, "%%%%%s\n", p.c_str());
}

void DataLogger::Log(int begin, int n, ...)
{
     if(!fp) return;
	va_list l;
	va_start(l, n);

	fprintf(fp, "%lf ", dtime());

	int i;
	for(i=0; i<begin; i++) {
		fprintf(fp, "NaN ");
	}

	for(; i<(n+begin); i++) {
		double pm = va_arg(l, double);
		fprintf(fp, "%lf ", pm);
	}

	for(; i<np; i++) {
		fprintf(fp, "NaN ");
	}

	fprintf(fp, "\n");

	va_end(l);
}

void DataLogger::Logv(int begin, int n, double *vec)
{
     if(!fp) return;
	fprintf(fp, "%lf ", dtime());
	int i;
	for(i=0; i<begin; i++) {
		fprintf(fp, "NaN ");
	}

	for(; i<(n+begin); i++) {
		fprintf(fp, "%lf ", vec[i-begin]);
	}

	for(; i<np; i++) {
		fprintf(fp, "NaN ");
	}

	fprintf(fp, "\n");
}

