#include "SDL/SDL_opengl.h"
#include <math.h>
#include "math_utils.h"
#include "gl_utils.h"
#include "warning.h"
#include "Target.h"

#include "AudioCore.h"

Target::Target(double _x, double _y, double _r, double _cr, double _cg, double _cb)
{
	x = _x;
	y = _y;
	radius = _r;
	r = _cr;
	g = _cg;
	b = _cb;
	geoms.resize(1);
	enable_movable = false;
	hit = false;
}

Target::~Target()
{
	gluDeleteQuadric(q);
	glDeleteTextures(1, &texture_id);
	fprintf(stderr, "Goodbye, cruel world!\n");
}

void Target::SetupODE(dWorldID world, dSpaceID world_space)
{
	geoms[0] = dCreateSphere(0, radius);
	dGeomSetPosition(geoms[0], x, y, radius);
	
	// following command makes it push-around-able
	if(enable_movable) {
		target_body = dBodyCreate(world);
		dMass m;
		dMassSetSphere(&m, 1, radius);
		dMassAdjust(&m, 0.2);
		dBodySetMass(target_body, &m);
		dGeomSetBody(geoms[0], target_body);
		dBodySetPosition(target_body, x, y, radius);
	}

	dSpaceAdd(world_space, geoms[0]);
}

void Target::SetupGL()
{
	q = gluNewQuadric();
	
	glGenTextures(1, &texture_id);
	glBindTexture(GL_TEXTURE_2D, texture_id);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, 128, 128, 0, GL_RGB, GL_UNSIGNED_BYTE, warning);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
}

void Target::Draw()
{
	if(!hit) {
		const dReal *pos, *rot;
		pos = dGeomGetPosition(geoms[0]);
		rot = dGeomGetRotation(geoms[0]);
		setTransform(pos, rot);

	//	glColor4f(r, g, b, 0.8);
	//	glBegin(GL_POLYGON);

	//	for(double n=-M_PI; n<M_PI; n+= 0.05*M_PI) {
	//		glVertex2f(sin(n)*radius*0.5, cos(n)*radius*0.5);
	//	}
	//	glEnd();
		glColor3f(r, g, b);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texture_id);
		gluQuadricTexture(q, true);
		gluSphere(q, radius, 24, 24);
		glDisable(GL_TEXTURE_2D);
	}
}

void Target::UpdatePositions()
{
	const dReal *p = dGeomGetPosition(geoms[0]);
	x = p[0];
	y = p[1];
}

int Target::UpdatePhysics()
{
	if(hit) {
		return OBJECT_ACTION_DELETE;
	}
	return OBJECT_ACTION_NONE;
}

int Target::HandleCollision(dGeomID, int, Object *other, dSurfaceParameters &surface)
{
	if(other->GetType() == OBJECT_GROUND) {
		surface.mu = 10;
		return COLLISION_HANDLED;
	} else {
		hit = true;
		surface.mu = 0;
		AudioCore::AC()->Play();
		return COLLISION_HANDLED_IGNORE;
	}

	return false;
}

