#include "SDL/SDL_opengl.h"
#include <math.h>
#include "Wall.h"

Wall::Wall(double _x1, double _y1, double _x2, double _y2, double _thickness,
	   double _r, double _g, double _b)
{
	x1 = _x1; x2 = _x2; y1 = _y1; y2 = _y2; thickness = _thickness;
	r = _r; g = _g; b = _b;

	angle = atan2(y2 - y1, x2 - x1);
	len = sqrt(pow(y2 - y1, 2) + pow(x2 - x1, 2));
	geoms.resize(1);
}

Wall::~Wall()
{
}

void Wall::Draw()
{
	glColor3f(r, g, b);
	glTranslated(x1, y1, 0);
	glRotated(angle * 180/M_PI - 90, 0, 0, 1);
#ifdef WIN32
	glBegin(GL_POLYGON);
	glVertex2f( thickness*0.5, 0);
	glVertex2f(-thickness*0.5, 0);
	glVertex2f(-thickness*0.5, len);
	glVertex2f( thickness*0.5, len);
	glEnd();
	glBegin(GL_POLYGON);
	glVertex3f(0, len, 0);
	glVertex3f(0, len, 0.4);
	glVertex3f(0,   0, 0.4);
	glVertex3f(0,   0, 0);
	glEnd();
#else
    /* Don't ask me why this doesn't work properly in Windows
       Objects just *disappear* when they hit the edge of the view
       frustum: closest reference to this that I could find online was
       at http://aig.cs.man.ac.uk/maverik/faq.php Question 13:
       
       My objects appear to be clipped to planes that don't coincide
       with the view frustum - I'm getting objects disappearing when
       they get near the edge of the window.
       
       Your view direction or view up vectors probably aren't
       normalized.
       
       ... but AFAIK, they are (check this though, pretty sure the
       identity matrix * glOrtho gives unit rotation vectors...
       */
	double v1[] = {thickness*0.5, -thickness*0.5, -thickness*0.5, thickness*0.5, thickness*0.5};
	double v2[] = {0, 0, len, len, 0};
	glBegin(GL_POLYGON);
	for(int i=0; i<4; i++) {
		glVertex3f(v1[i], v2[i], 0);
		glVertex3f(v1[i], v2[i], 0.4);
		glVertex3f(v1[i+1], v2[i+1], 0.4);
		glVertex3f(v1[i+1], v2[i+1], 0);
	}
	for(int i=0; i<4; i++) {
		glVertex3f(v1[i], v2[i], 0);
	}
	for(int i=0; i<4; i++) {
		glVertex3f(v1[i], v2[i], 0.4);
	}
	glEnd();
#endif
}

void Wall::SetupODE(dWorldID, dSpaceID space)
{
	geoms[0] = dCreateBox(space, len, thickness, 1);
	dGeomSetPosition(geoms[0], (x1 + x2)/2, (y1 + y2)/2, 0.4);
	dMatrix3 R;
	dRFromAxisAndAngle(R, 0, 0, 1, angle);
	dGeomSetRotation(geoms[0], R);
}

void Wall::UpdatePositions()
{
	x = (x1+x2)/2;
	y = (y1+y2)/2;
	z = 0;
	theta = angle;
}

int Wall::HandleCollision(dGeomID, int wo, Object *other, dSurfaceParameters &)
{
	if(other->GetType() == OBJECT_GROUND) return COLLISION_HANDLED_IGNORE;
	if(other->GetType() == OBJECT_WALL && wo == 1) return COLLISION_HANDLED_IGNORE;
	return COLLISION_UNHANDLED;
}

