#ifndef __core_h__
#define __core_h__

#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"
#include <ode/ode.h>
#include "UCInterface.h"
#include "Object.h"
#include "Robot.h"
#include "Ground.h"
#include "ODEInterface.h"
#include "AudioCore.h"
#include "DataLogger.h"
#include "WorldClock.h"

using namespace std;

class UCInterface;

class SimCore {
friend class RobotIF;
public:
	SimCore();
	~SimCore();

	void Start();

	void LoadWallsFile(string path);
	void LoadTargetsFile(string path);
	void SetWindowSize(int x = 640, int y = 480);
	void SetLoggerFile(string path);
	void GetRobotPosVec(double *);
	void SetUserParamsArg(string params);

private:
	/* SDL/GL stuff */
	void SDL_GL_Setup();
	SDL_Surface *screen;
	int sW, sH;
	bool done;
	void SetWindowPriTitle(string);
	void SetWindowSecondaryTitle(string);
	string pri_title, sec_title;
	void UpdateTitle();

	double p2m;
	double scaleMult;
	
	void SetupCamera();
	void ToggleCameraMode();
	int camera_mode;
	double camx, camy;
	bool WorldMousePosition();

	double mousex, mousey;
	double worldx, worldy;

	WorldClock *wc;

	/* Object stuff */
	list<Object *> objects;
	Robot *robot;
	Ground *ground;

	map<dGeomID, Object *> geom_map;

	void DrawObjects();

	void SetupWallGeoms();
	void SetupTargetGeoms();

	/* User code interface */
	UCInterface *user_if;
	string ucparams;

	/* ODE */
	ODEInterface *ODEif;

	/* audio */
	AudioCore *ac;

	/* datalogger */
	string logfile;
};

enum {
	CAMERA_FOLLOW_ROBOT_TS,
	CAMERA_FOLLOW_ROBOT_TANDR,
	CAMERA_FIXED,
	CAMERA_FOLLOW_3D_TRACK_X,
	CAMERA_FOLLOW_3D_TRACK_Y,
	CAMERA_FOLLOW_3D_TRACK_XY,
};

#endif
